/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio;

import java.io.IOException;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import kr.sohalee.hangastudio.core.cache.CacheMap;
import kr.sohalee.hangastudio.core.system.SSLInitializer;
import kr.sohalee.hangastudio.model.commcode.CommCodeCacheMap;

import org.apache.log4j.Logger;

/**
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
@SuppressWarnings("serial")
public class HgInitializer extends HttpServlet {

	private static Logger logger = Logger.getLogger(HgInitializer.class);

	@Override
	public void destroy() {

		if (logger.isInfoEnabled()) {
			logger.info("[HG] initialization was destroyed");
		}
	}

	@Override
	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		response.sendError(HttpServletResponse.SC_FORBIDDEN);
	}

	@Override
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		response.sendError(HttpServletResponse.SC_FORBIDDEN);
	}

	@Override
	public void init(ServletConfig servletConfig) throws ServletException {

		super.init(servletConfig);

		if (logger.isInfoEnabled()) {
			logger.info("[HG] initialization started");
		}

		try {
			// SSL 사용 가능하게 초기화
			SSLInitializer.init();

			// 공통 코드 초기화
			CacheMap.setServletContext(servletConfig.getServletContext());
			CommCodeCacheMap.getCommCodeList(null);
			CacheMap.setServletContext(null);
		} catch (Exception e) {
			logger.error("[HG] initialization failed");
			throw new ServletException(e);
		}

		if (logger.isInfoEnabled()) {
			logger.info("[HG] initialization successfully");
		}
	}
}
