/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.core.cache;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletContext;

import org.apache.struts2.ServletActionContext;
import org.springframework.context.ApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import com.ibatis.sqlmap.client.SqlMapClient;

/**
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
@SuppressWarnings("unchecked")
public abstract class CacheMap {

	private static ServletContext staticServletContext = null;

	/**
	 * CacheMap에서 SqlMapClient를 가져오기위한 servletContext를 포함한다.<br />
	 * Servlet Container가 구동할때 initialize 할때만 셋팅한다.<br />
	 * 모든것이 로딩되고 실제 웹페이지를 통한 서비스 중에서는 null로한다.
	 * 
	 * @param servletContext
	 */
	public static void setServletContext(ServletContext servletContext) {
		staticServletContext = servletContext;
	}

	private final Map<String, Object> cacheMap = Collections
			.synchronizedMap(new HashMap<String, Object>());

	public abstract void initCache() throws Exception;

	public Object get(String key) {
		return cacheMap.get(key);
	}

	public void put(String key, Object value) {

		remove(key);
		cacheMap.put(key, value);
	}

	public void remove(String key) {

		if (cacheMap.containsKey(key)) {
			cacheMap.remove(key);
		}
	}

	public void clear() {
		cacheMap.clear();
	}

	protected SqlMapClient getSqlMapClient() {

		ServletContext servletContext = ServletActionContext
				.getServletContext();
		if (servletContext == null) {
			servletContext = staticServletContext;
		}

		ApplicationContext appContext = WebApplicationContextUtils
				.getWebApplicationContext(servletContext);

		servletContext = null;

		return (SqlMapClient) appContext.getBean("sqlMapClient");
	}
}
