/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.core.cache;

import com.opensymphony.oscache.base.NeedsRefreshException;
import com.opensymphony.oscache.general.GeneralCacheAdministrator;
import com.opensymphony.oscache.util.ClassLoaderUtil;

/**
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public class CacheServiceImplOSCache implements CacheService {

	private static final GeneralCacheAdministrator osCache = new GeneralCacheAdministrator();

	private CacheMap newInstance(String className) throws Exception {

		Class classObj = ClassLoaderUtil.loadClass(className, this.getClass());
		CacheMap cacheMap = (CacheMap) classObj.newInstance();
		cacheMap.initCache();

		return cacheMap;
	}

	public CacheMap get(String key) throws Exception {

		try {
			return (CacheMap) osCache.getFromCache(key);
		} catch (NeedsRefreshException ne) {

			try {

				CacheMap cacheMap = newInstance(key);
				osCache.putInCache(key, cacheMap);

				return cacheMap;
			} catch (Exception e) {
				osCache.cancelUpdate(key);
				// 캐쉬에는 시스템적인 내용만 담을것이므로 캐시를 제대로 만들지 못하면 시스템 에러로 간주한다.
				throw e;
			}
		}
	}

	public Object get(String key, String cacheMapKey) throws Exception {

		CacheMap cacheMap = get(key);
		if (cacheMap != null) {
			return cacheMap.get(cacheMapKey);
		}

		return null;
	}

	public void remove(String key) throws Exception {

		if (get(key) != null) {
			osCache.removeEntry(key);
		}
	}

	public void flush(String key) throws Exception {
		osCache.flushEntry(key);
	}

	public void flushAll() throws Exception {
		osCache.flushAll();
	}

}
