/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.core.taglib;

import java.io.Serializable;
import java.net.URLEncoder;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyTagSupport;

import kr.sohalee.hangastudio.core.type.PageNavigation;

import org.apache.log4j.Logger;

@SuppressWarnings("serial")
public class PageNaviTag extends BodyTagSupport implements Serializable {

	private static Logger logger = Logger.getLogger(PageNaviTag.class);

	private static final String LINK_FORMAT = "<a %s %s>%s</a>&nbsp;";

	private static final String PREV_PACK_TAG = "&lt;&lt;PrevPack";

	private static final String NEXT_PACK_TAG = "NextPack&gt;&gt;";

	public static final String REQUEST_SCOPE = "request";

	public static final String SESSION_SCOPE = "session";

	private String jsName = null;

	private String url = null;

	private String naviScope = REQUEST_SCOPE;

	private String naviValue = null;

	private PageNavigation pageNavi = null;

	private String cssClass = null;

	private String cssClassSide = null;

	private String cssClassPage = null;

	private String cssClassOnPage = null;

	private String cssClassDisable = null;

	private String prevPackTag = null;

	private String nextPackTag = null;

	private String replaceUrl = null;

	private String encoding = null;

	private void initProperty() {
		jsName = null;
		url = null;
		naviScope = REQUEST_SCOPE;
		naviValue = null;
		pageNavi = null;
		cssClass = null;
		cssClassSide = null;
		cssClassPage = null;
		cssClassOnPage = null;
		cssClassDisable = null;
		prevPackTag = null;
		nextPackTag = null;
		replaceUrl = null;
		encoding = null;
	}

	private boolean isEmpty(String str) {
		return str == null || str.trim().equals("") ? true : false;
	}

	private Object getContextAttribute(String scope, String name) {

		Object result = null;
		if (scope.equals(REQUEST_SCOPE)) {
			result = pageContext.getRequest().getAttribute(name);
		} else if (scope.equals(SESSION_SCOPE)) {
			result = pageContext.getSession().getAttribute(name);
		}

		return result;
	}

	private void setPageNavi() throws JspException {

		pageNavi = (PageNavigation) getContextAttribute(naviScope, naviValue);
		if (pageNavi == null) {
			throw new JspException(
					"naviScope에 지정된 위치에서 naviValue Attribute를 찾을수 없습니다. Scope :"
							+ naviScope);
		}
	}

	private void validationParameter() throws JspException {

		boolean valid = false;
		String msg = null;

		if (isEmpty(url) && isEmpty(jsName)) {
			msg = "url(Body에 정의) 또는 jsName중 하나는 반드시 정의해 주셔야 합니다.";
		} else if (isEmpty(naviScope)
				|| (!naviScope.equals(REQUEST_SCOPE) && !naviScope
						.equals(SESSION_SCOPE))) {
			msg = "naviScope는 \"request\"또는 \"session\"중 하나여야합니다";
		} else if (isEmpty(naviValue)) {
			msg = "naviValue를 지정하셔야 합니다.";
		} else {
			valid = true;
		}

		if (valid == false) {
			throw new JspException(msg);
		}
	}

	private void setUrl() throws Exception {

		if (!isEmpty(replaceUrl)) {

			String[] replaceArray = replaceUrl.split("[;]");
			for (String setStr : replaceArray) {

				String[] setArray = setStr.split("[:]");
				if (setArray.length < 2) {
					continue;
				}

				String scope = REQUEST_SCOPE;
				if (setArray.length > 2) {
					scope = setArray[3];
				}

				Object attributeObj = getContextAttribute(scope, setArray[1]);
				url = url.replaceAll(setArray[0], URLEncoder.encode(
						attributeObj.toString(), getEncoding()));
			}
		}
	}

	@Override
	public int doStartTag() throws JspException {
		try {

			if (!isEmpty(jsName)) {

				validationParameter();

				setPageNavi();
				pageContext.getOut().write(getPagingHtmlTag());

				return SKIP_BODY;
			}
		} catch (Exception e) {

			logger.error(e, e);
			throw new JspException(e);
		}

		return EVAL_BODY_BUFFERED;
	}

	@Override
	public int doEndTag() throws JspException {
		return EVAL_PAGE;
	}

	@Override
	public int doAfterBody() throws JspException {

		try {

			url = getBodyContent().getString().trim();
			validationParameter();

			setUrl();
			setPageNavi();
			getPreviousOut().print(getPagingHtmlTag());
		} catch (Exception e) {

			logger.error(e, e);
			throw new JspException(e);
		}

		return SKIP_BODY;
	}

	private String getCssTag(String css) {

		String cssValue = css;
		if (isEmpty(css)) {
			cssValue = cssClass;
		}

		return isEmpty(cssValue) ? "" : "class=\"" + cssValue + "\"";
	}

	private String getCssSide() {
		return getCssTag(cssClassSide);
	}

	private String getCssDisable() {
		return getCssTag(cssClassDisable);
	}

	private String getCssPage(int page) {

		String css = cssClassPage;
		if (page == pageNavi.getCurrPageNo().intValue()) {
			css = cssClassOnPage;
		}

		return getCssTag(css);
	}

	private int getPageCount() {

		int pageCount = (int) Math.ceil((float) pageNavi.getItemTotalCnt()
				.intValue()
				/ pageNavi.getItemCntPerPage().intValue());

		return pageCount >= 1 ? pageCount : 1;
	}

	private int getPageStart() {

		int pagePackIndex = (int) Math.ceil((float) pageNavi.getCurrPageNo()
				.intValue()
				/ pageNavi.getPageScale().intValue()) - 1;

		return (pagePackIndex * pageNavi.getPageScale().intValue()) + 1;
	}

	private int getPageEnd(int pageStart, int pageCount) {

		int pageEnd = pageStart + pageNavi.getPageScale().intValue() - 1;

		return pageEnd <= pageCount ? pageEnd : pageCount;
	}

	private String getPrevPackTag() {
		return isEmpty(prevPackTag) ? PREV_PACK_TAG : prevPackTag;
	}

	private String getNextPackTag() {
		return isEmpty(nextPackTag) ? NEXT_PACK_TAG : nextPackTag;
	}

	private String getHrefAttribute(int page) {

		String hrefAttribute = "";

		if (isEmpty(jsName)) {
			hrefAttribute = "href=\"" + url + page + "\"";
		}

		return hrefAttribute;
	}

	private String getTagAttributes(int page, String css) {

		String tagAttributes = css;

		if (!isEmpty(jsName)) {
			tagAttributes += " onclick=\"" + jsName + "(" + page + ");\"";
		}

		return tagAttributes;
	}

	private String getPrevTag(int pageStart) {

		String hrefAttribute = "";
		String tagAttributes = getCssDisable();

		int prevPage = pageStart - 1;
		if (prevPage > 0) {
			hrefAttribute = getHrefAttribute(prevPage);
			tagAttributes = getTagAttributes(prevPage, getCssSide());
		}

		return String.format(LINK_FORMAT, hrefAttribute, tagAttributes,
				getPrevPackTag());
	}

	private String getNextTag(int pageEnd, int pageCount) {

		String hrefAttribute = "";
		String tagAttributes = getCssDisable();

		int nextPage = pageEnd + 1;
		if (nextPage <= pageCount) {
			hrefAttribute = getHrefAttribute(nextPage);
			tagAttributes = getTagAttributes(nextPage, getCssSide());
		}

		return String.format(LINK_FORMAT, hrefAttribute, tagAttributes,
				getNextPackTag());
	}

	private String getPagingHtmlTag() throws JspException {

		int pageCount = getPageCount();
		StringBuffer sbuf = new StringBuffer();

		int pageStart = getPageStart();
		int pageEnd = getPageEnd(pageStart, pageCount);

		sbuf.append(getPrevTag(pageStart));
		for (int i = pageStart; i <= pageEnd; i++) {
			sbuf.append(String.format(LINK_FORMAT, getHrefAttribute(i),
					getTagAttributes(i, getCssPage(i)), i));
		}
		sbuf.append(getNextTag(pageEnd, pageCount));

		return sbuf.toString();
	}

	@Override
	public void release() {

		super.release();
		initProperty();
	}

	public void setJsName(String jsName) {
		this.jsName = jsName;
	}

	public void setCssClass(String cssClass) {
		this.cssClass = cssClass;
	}

	public void setCssClassOnPage(String cssClassOnPage) {
		this.cssClassOnPage = cssClassOnPage;
	}

	public void setCssClassPage(String cssClassPage) {
		this.cssClassPage = cssClassPage;
	}

	public void setCssClassSide(String cssClassSide) {
		this.cssClassSide = cssClassSide;
	}

	public void setCssClassDisable(String cssClassDisable) {
		this.cssClassDisable = cssClassDisable;
	}

	public void setNextPackTag(String nextPackTag) {
		this.nextPackTag = nextPackTag;
	}

	public void setPrevPackTag(String prevPackTag) {
		this.prevPackTag = prevPackTag;
	}

	public void setNaviScope(String naviScope) {
		this.naviScope = naviScope;
	}

	public void setNaviValue(String naviValue) {
		this.naviValue = naviValue;
	}

	public void setReplaceUrl(String replaceUrl) {
		this.replaceUrl = replaceUrl;
	}

	public String getEncoding() {

		String enc = encoding;
		if (isEmpty(enc)) {
			enc = System.getProperty("file.encoding");
		}

		return encoding;
	}

	public void setEncoding(String encoding) {
		this.encoding = encoding;
	}

}