/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.core.util;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * HTML 관련 유틸리티<br>
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public class HtmlUtil {

	private static String getRegexTag(String tag) {
		return "<" + tag + "[^>]*>|</" + tag + "[^>]*>";
	}

	private static String getRegexTagInnerBody(String tag) {
		return "<" + tag + "[^>]*>.*</" + tag + "[^>]*>";
	}

	private static String removePattern(String text, String regex) {

		Pattern pattern = Pattern.compile(regex, Pattern.DOTALL
				| Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
		Matcher matcher = pattern.matcher(text);
		if (matcher.find()) {
			return matcher.replaceAll("");
		}

		return text;
	}

	public static String removeTag(String text, String tag, boolean removeBody) {

		String result = text;

		if (removeBody) {
			result = removePattern(result, getRegexTagInnerBody(tag));
		}

		return removePattern(result, getRegexTag(tag));
	}

	/** 해당 문자가 개행문자(\n)인경우 &lt;br /&gt;을 추가하여 반환. (\r)제거 */
	private static String lineBreakConvertHtml(char ch) {

		String result = null;

		switch (ch) {

			case '\n': {
				result = "<br />\n";
				break;
			}
			case '\r': {
				result = "";
				break;
			}
			default: {
				result = String.valueOf(ch);
				break;
			}
		}

		return result;
	}

	/**
	 * Html 특수문자를 Html 엔터티로 변환환다.<br />
	 * Html 소스를 웹브라우저에 표시하기 위해 변환합니다.
	 * 
	 * @param text
	 * @param innerLineBreak
	 *            true시 개행부분에 &lt;br /&gt;를 붙여줍니다.
	 * @return
	 */
	public static String toText(String text, boolean innerLineBreak) {

		if (StringUtil.isEmpty(text)) {
			return "";
		}

		StringBuffer sBuf = new StringBuffer();

		int length = text.length();
		for (int i = 0; i < length; i++) {

			char tmp = text.charAt(i);
			switch (tmp) {

				case '<': {
					sBuf.append("&lt;");
					break;
				}
				case '>': {
					sBuf.append("&gt;");
					break;
				}
				case '&': {
					sBuf.append("&amp;");
					break;
				}
				default: {

					if (innerLineBreak) {
						sBuf.append(lineBreakConvertHtml(tmp));
					} else {
						sBuf.append(tmp);
					}
					break;
				}
			}
		}

		return sBuf.toString();
	}

	/**
	 * Text의 개행에 Html태그 &ltbr /&gt를 추가하여 반환한다.<br />
	 * Text를 웹브라우저에 표기할때 개행이 되어 표시하기 위해 변환합니다.
	 * 
	 * @param text
	 * @param innerLineBreak
	 *            true시 개행부분에 &lt;br /&gt;를 붙여줍니다.
	 * @return
	 */
	public static String toInnerLineBreak(String text) {

		if (StringUtil.isEmpty(text)) {
			return "";
		}

		StringBuffer sBuf = new StringBuffer();

		int length = text.length();
		for (int i = 0; i < length; i++) {

			char tmp = text.charAt(i);
			sBuf.append(lineBreakConvertHtml(tmp));
		}

		return sBuf.toString();
	}
}
