/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.core.util;

/**
 * 정수형 관련 Util
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public class IntegerUtil {

	/**
	 * Integer의 intValue를 비교한다. 파라미터의 값이 하나라도 null인경우 false를 반환한다.
	 * 
	 * @param arg1
	 * @param arg2
	 * @return
	 */
	public static boolean compareIntValue(Integer arg1, Integer arg2) {

		if (arg1 == null || arg2 == null) {
			return false;
		}

		return arg1.intValue() == arg2.intValue();
	}

	/**
	 * 기본형 int로 값을 반환한다. null일경우 defaultValue를 반환한다.
	 * 
	 * @param integer
	 * @param defaultValue
	 *            integer가 null일 경우 반환할 기본 값
	 * @return
	 */
	public static int intValue(Integer integer, int defaultValue) {

		if (integer == null) {
			return defaultValue;
		}

		return integer.intValue();
	}

	/**
	 * 기본형 int로 값을 반환한다. null일 경우 -1을 반환한다.
	 * 
	 * @param integer
	 * @return
	 */
	public static int intValue(Integer integer) {
		return intValue(integer, -1);
	}

	/**
	 * 기본형 int로 값을 반환한다. null일경우 defaultValue를 반환한다.
	 * 
	 * @param str
	 * @param defaultValue
	 *            integer가 null일 경우 반환할 기본 값
	 * @return
	 */
	public static int intValue(String str, int defaultValue) {

		if (str == null) {
			return intValue((Integer) null);
		}

		return intValue(Integer.valueOf(str));
	}

	/**
	 * 기본형 int로 값을 반환한다. null일 경우 -1을 반환한다.
	 * 
	 * @param str
	 * @return
	 */
	public static int intValue(String str) {
		return intValue(str, -1);
	}
}
