/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.core.util;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.activation.DataHandler;
import javax.mail.Address;
import javax.mail.BodyPart;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.Store;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMultipart;
import javax.mail.internet.MimeUtility;

public class JavaMailUtil {

	public static final String ENCODING_FOR_UNKNOWN = "EUC-KR";

	public static final String TEXT_PLAIN = "text/plain";

	public static final String TEXT_HTML = "text/html";

	public static final String MULTIPART_ALTERNATIVE = "multipart/alternative";

	public static final String MESSAGE_RFC822 = "message/rfc822";

	public static final String ALTERNATIVE = "alternative";

	private static final String SSL_SOCKET_FACTORY_CLASS = "javax.net.ssl.SSLSocketFactory";

	public static void openFolder(Object folder, int mode) throws Exception {

		Folder folderTmp = (Folder) folder;
		if (!folderTmp.isOpen()) {
			folderTmp.open(mode);
		}
	}

	public static Folder closeFolder(Object folder) throws Exception {

		Folder folderTmp = (Folder) folder;
		if (folderTmp != null && folderTmp.isOpen()) {
			folderTmp.close(false);
		}

		return null;
	}

	private static String decodeUnknownFormat(String text, String encoding) {

		try {

			String enc = encoding;
			if (StringUtil.isEmpty(enc)) {

				if (StringUtil.isEmpty(ENCODING_FOR_UNKNOWN)) {
					enc = System.getProperty("file.encoding");
				} else {
					enc = ENCODING_FOR_UNKNOWN;
				}
			}

			return new String(text.getBytes("8859_1"), enc);
		} catch (UnsupportedEncodingException e) {
			return text;
		}
	}

	public static String decodeText(String text, String subCharset) {

		if (text == null) {
			return null;
		}

		// 결과물
		StringBuffer sBuf = new StringBuffer();

		// 여러줄로 나누어 오는 것이 있으므로 분리해서 Collection에 담는다.
		String tmp = text.replaceAll("\r|\n", "");

		Pattern pattern = Pattern.compile("=\\?[^?]*\\?[^?]*\\?[^?]*\\?=");

		while (true) {

			String before = null;

			Matcher matcher = pattern.matcher(tmp);
			// 인코딩된 문자열이 있는경우.
			if (matcher.find()) {

				int start = matcher.start();
				// 인코딩된 문자열 앞에 비 인코딩 문자열이 있는 경우
				if (start > 0) {

					before = tmp.substring(0, start);
					sBuf.append(decodeUnknownFormat(before, subCharset));

					before = null;
				}

				String group = matcher.group();
				// 인코딩된 문자열을 저장
				try {
					sBuf.append(MimeUtility.decodeText(group));
				}
				// 지원하지 않는 Encoding인경우 지정 또는 시스템 Encoding으로 강제 변환한다.
				catch (UnsupportedEncodingException e) {

					// ?를 구분자로 텍스트를 자른다.
					String[] otherArray = group.split("[?]");
					// 4번째가 제목 텍스트이다. 위의 정규식에 의하면 무조건 존재한다.
					sBuf.append(decodeUnknownFormat(otherArray[3], subCharset));
				}

				tmp = tmp.substring(matcher.end());
			}
			// 인코딩된 문자열이 없는 경우 (멀티바이트 문자열인경우 비표준인경우이다)
			else {

				sBuf.append(decodeUnknownFormat(tmp, subCharset));
				tmp = "";
			}

			if (tmp.length() == 0) {
				break;
			}
		}

		return sBuf.toString();
	}

	public static String getHeaderFirst(Part part, String name)
			throws Exception {

		String result = null;

		String[] array = part.getHeader(name);
		if (array != null && array.length > 0) {
			result = array[0];
		}

		return result;
	}

	public static String[] getFrom(Message message) throws Exception {

		String[] from = { null, null };

		Address[] address = message.getFrom();
		if (address != null && address.length > 0) {

			String tmp = decodeText(address[0].toString(),
					getContentCharset(message));

			Pattern pattern = Pattern.compile("<.*@.*>");
			Matcher matcher = pattern.matcher(tmp);
			// 이름과 메일주소로 이루어진 From인경우
			if (matcher.find()) {

				from[0] = tmp.substring(0, matcher.start())
						.replaceAll("\"", "").trim();
				from[1] = matcher.group().replaceAll("^<|>$", "").trim();
			}
			// 메일 주소만 있는경우
			else {

				// 두개의 내용을 똑같이 만든다.
				from[0] = tmp.trim();
				from[1] = from[0];
			}

		}

		return from;
	}

	public static String getContentMimeType(Part part) throws Exception {

		if (part == null || part.getContentType() == null) {
			return null;
		}

		String[] strArray = part.getContentType().split("[;]");

		return strArray[0].trim();
	}

	public static String getContentCharset(Part part) throws Exception {

		if (part == null || part.getContentType() == null) {
			return null;
		}

		String charset = null;

		Pattern pattern = Pattern
				.compile("^charset=", Pattern.CASE_INSENSITIVE);

		String[] strArray = part.getContentType().split("[;]");
		for (String str : strArray) {

			Matcher matcher = pattern.matcher(str.trim());
			if (matcher.find()) {

				charset = matcher.replaceAll("").replaceAll("\"", "");
				break;
			}
		}

		return charset;
	}

	/**
	 * Multipart로부터 텍스트(메일 메세지 본문)를 추출하여 반환한다.
	 * 
	 * @param multipart
	 * @return BodyPart
	 * @throws Exception
	 */
	public static BodyPart getTextBodyPart(Multipart multipart)
			throws Exception {

		BodyPart bodyPart = null;

		for (int i = 0; i < multipart.getCount(); i++) {

			bodyPart = multipart.getBodyPart(i);
			if (!bodyPart.isMimeType("text/*")) {
				continue;
			}

			if (bodyPart.isMimeType("text/html")) {
				break;
			}
		}

		return bodyPart;
	}

	public static BodyPart getBodyPart(Multipart multipart, int[] mpartPath)
			throws Exception {

		Part part = null;

		for (int i = 0; i < mpartPath.length; i++) {

			if (i > 0) {
				Part subPart = (Part) part.getContent();
				multipart = (Multipart) subPart.getContent();
			}

			int index = mpartPath[i];
			part = multipart.getBodyPart(index);
		}

		return (BodyPart) part;
	}

	public static BodyPart getBodyPart(Multipart multipart, String[] mpartPath)
			throws Exception {

		// 문자열의 배열로 되어있는 multipart의 path를 숫자로 변환한다.
		int[] pathArray = new int[mpartPath.length];
		for (int i = 0; i < mpartPath.length; i++) {
			pathArray[i] = Integer.parseInt(mpartPath[i]);
		}

		return getBodyPart(multipart, pathArray);
	}

	public static BodyPart getContentBodyPart(String content, String mimeType,
			String charset) throws Exception {

		String charsetFinal = "";
		if (!StringUtil.isEmpty(charset)) {
			charsetFinal = "; charset=\"" + charset + "\"";
		}

		MimeBodyPart bodyPart = null;

		// text/plain
		String textContentType = TEXT_PLAIN + charsetFinal;
		MimeBodyPart textPart = new MimeBodyPart();
		textPart.setContent(content, textContentType);
		textPart.setHeader("Content-Type", textContentType);

		// text/html
		String htmlContentType = TEXT_HTML + charsetFinal;
		MimeBodyPart htmlPart = new MimeBodyPart();

		if (mimeType.equalsIgnoreCase(MULTIPART_ALTERNATIVE)) {

			// alternative인 경우 html에는 내용의 개행에 <br />태그를 함께 넣는다.
			htmlPart.setContent(HtmlUtil.toInnerLineBreak(content),
					htmlContentType);

			MimeMultipart alternativePart = new MimeMultipart(ALTERNATIVE);
			alternativePart.addBodyPart(textPart);
			alternativePart.addBodyPart(htmlPart);

			bodyPart = new MimeBodyPart();
			bodyPart.setContent(alternativePart);
		} else if (mimeType.equalsIgnoreCase(TEXT_HTML)) {

			htmlPart.setContent(content, htmlContentType);
			bodyPart = htmlPart;
		} else {
			bodyPart = textPart;
		}

		htmlPart.setHeader("Content-Type", htmlContentType);

		textContentType = null;
		htmlContentType = null;
		textPart = null;
		htmlPart = null;

		return bodyPart;
	}

	public static BodyPart getMessageRfc822BodyPart(Message message)
			throws Exception {

		DataHandler handler = new DataHandler(message, MESSAGE_RFC822);

		MimeBodyPart bodyPart = new MimeBodyPart();
		bodyPart.setDataHandler(handler);

		return bodyPart;
	}

	public static void setPropertiesForSsl(Properties destProps,
			String protocol, String port) {

		String portKey = "mail." + protocol + ".port";
		destProps.remove(portKey);
		destProps.put(portKey, port);

		destProps.put("mail." + protocol + ".socketFactory.class",
				SSL_SOCKET_FACTORY_CLASS);
		destProps.put("mail." + protocol + ".socketFactory.fallback", "false");
		destProps.put("mail." + protocol + ".socketFactory.port", port);
	}

	public static void attachPutOutputStream(Store connectedStore,
			String folderName, int messageNo, String[] mpartPath,
			OutputStream outputStream) throws Exception {

		if (!connectedStore.isConnected()) {
			throw new Exception("store not connected");
		}
		Folder folder = null;

		try {

			folder = connectedStore.getDefaultFolder();
			folder = folder.getFolder(folderName);
			folder.open(Folder.READ_ONLY);

			Message message = folder.getMessage(messageNo);

			BodyPart part = null;
			if (message.isMimeType("multipart/*")) {

				Multipart multipart = (Multipart) message.getContent();
				part = JavaMailUtil.getBodyPart(multipart, mpartPath);

				multipart = null;
			}

			BufferedInputStream bis = new BufferedInputStream(part
					.getInputStream());
			BufferedOutputStream bos = null;

			byte[] buff = new byte[2 * 1024];
			int readCount = 0;

			try {

				bos = new BufferedOutputStream(outputStream);

				while ((readCount = bis.read(buff)) != -1) {
					bos.write(buff, 0, readCount);
					bos.flush();
				}
			} finally {
				if (bis != null) {
					bis.close();
					bis = null;
				}
				if (bos != null) {
					bos.close();
					bos = null;
				}
			}
		} finally {
			folder = closeFolder(folder);
		}
	}

}
