/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.core.util;

/**
 * 문자 관련 Util
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public class StringUtil {

	private static final String UTF8_ENCODING = "UTF-8";

	public static boolean isEmpty(String str) {

		if (str == null) {
			return true;
		}

		return str.trim().equals("");
	}

	public static boolean isEquals(String str, String value) {

		if (str == null) {
			return false;
		}

		return str.equals(value);
	}

	public static boolean isEqualsIgnoreCase(String str, String value) {

		if (str == null) {
			return false;
		}

		return str.equalsIgnoreCase(value);
	}

	public static String toNotNull(String str) {

		if (str == null) {
			return "";
		}

		return str;
	}

	public static String toExcel(String text) {

		if (isEmpty(text) == true) {
			return "";
		}

		StringBuffer sBuf = new StringBuffer();

		int length = text.length();
		for (int i = 0; i < length; i++) {

			char tmp = text.charAt(i);
			if (tmp != '\r') {
				sBuf.append(tmp);
			}
		}

		return sBuf.toString();
	}

	public static int lineCount(String text) {

		if (isEmpty(text) == true) {
			return 0;
		}

		int count = 1;

		int length = text.length();
		for (int i = 0; i < length; i++) {

			char tmp = text.charAt(i);

			if (tmp == '\n') {
				count++;
			}
		}

		return count;
	}

	private static final int UTF8_LENGTH_INFO_BYTE_LENGTH_ID = 0;

	private static final int UTF8_LENGTH_INFO_CURRENT_BYTE_ID = 1;

	private static final int UTF8_LENGTH_INFO_DISPLAY_LENGTH = 2;

	private static int[] getUTF8LengthInfo(String str, int size,
			int utf8DisplaySize) throws Exception {

		int byteLength = 0;
		int currentByte = 0;
		int displayLength = 0;

		byte[] byteArray = str.getBytes(UTF8_ENCODING);

		while (byteLength < byteArray.length) {

			byte tmp = byteArray[byteLength];

			if (tmp == 9 || tmp == 10 || (tmp >= 32 && tmp <= 126)) {

				byteLength++;
				currentByte = 1;
				displayLength++;
			} else if (tmp >= -62 && tmp <= -33) { // tmp >= 194 && tmp <= 223

				byteLength += 2;
				currentByte = 2;
				displayLength += utf8DisplaySize;
			} else if (tmp >= -32 && tmp <= -17) { // tmp >= 224 && tmp <= 239

				byteLength += 3;
				currentByte = 3;
				displayLength += utf8DisplaySize;
			} else if (tmp >= -16 && tmp <= -9) { // tmp >= 240 && tmp <= 247

				byteLength += 4;
				currentByte = 4;
				displayLength += utf8DisplaySize;
			} else if (tmp >= -8 && tmp <= -5) { // tmp >= 248 && tmp <= 251

				byteLength += 5;
				currentByte = 5;
				displayLength += utf8DisplaySize;
			} else if (tmp >= -4 && tmp <= -3) { // tmp >= 252 && tmp <= 253

				byteLength += 6;
				currentByte = 6;
				displayLength += utf8DisplaySize;
			} else {
				byteLength++;
			}

			if (size > 0 && displayLength >= size) {
				break;
			}
		}

		int[] lengthInfoArray = new int[3];
		lengthInfoArray[UTF8_LENGTH_INFO_BYTE_LENGTH_ID] = byteLength;
		lengthInfoArray[UTF8_LENGTH_INFO_CURRENT_BYTE_ID] = currentByte;
		lengthInfoArray[UTF8_LENGTH_INFO_DISPLAY_LENGTH] = displayLength;

		byteArray = null;

		return lengthInfoArray;
	}

	public static String UTF8Substring(String str, int size,
			int utf8DisplaySize, String endStr) throws Exception {

		int[] lengthInfoArray = getUTF8LengthInfo(str, size, utf8DisplaySize);
		int byteLength = lengthInfoArray[UTF8_LENGTH_INFO_BYTE_LENGTH_ID];
		int currentByte = lengthInfoArray[UTF8_LENGTH_INFO_CURRENT_BYTE_ID];
		int displayLength = lengthInfoArray[UTF8_LENGTH_INFO_DISPLAY_LENGTH];

		if (displayLength > size) {
			byteLength -= currentByte;
		}

		byte[] byteArray = str.getBytes(UTF8_ENCODING);
		byte[] newByte = new byte[byteLength];
		for (int i = 0; i < newByte.length; i++) {
			newByte[i] = byteArray[i];
		}

		String resultStr = new String(newByte, UTF8_ENCODING);
		if (byteLength < byteArray.length) {
			resultStr += endStr;
		}

		byteArray = null;
		newByte = null;

		return resultStr;
	}

	public static String twoByteSubstring(String str, int size, String endStr,
			String encoding) throws Exception {

		byte[] byteArray = str.getBytes(encoding);

		if (size < byteArray.length) {

			int twoByteChar = 0;
			for (int i = 0; i < size; i++) {

				if (byteArray[i] < 0) {
					twoByteChar++;
				}
			}

			int finalSize = size;
			if (twoByteChar % 2 == 1) {
				finalSize--;
			}

			byte[] newByte = new byte[finalSize];
			for (int i = 0; i < newByte.length; i++) {
				newByte[i] = byteArray[i];
			}

			return new String(newByte, encoding) + endStr;
		} else {
			return str;
		}
	}

	public static String korSubstring(String str, int size, String endStr) {

		// if (isEqualsIgnoreCase(System.getProperty("file.encoding"),
		// UTF8_ENCODING)) {
		// return UTF8Substring(str, size, 2, endStr);
		// } else {
		// return twoByteSubstring(str, size, endStr, "euc-kr");
		// }

		int count = 0;
		int i = 0;
		for (i = 0; i < str.length(); i++) {

			int codeval = str.codePointAt(i);
			if (codeval < 128) {
				count++;
			} else {
				count += 2;
			}

			if (count > size) {
				break;
			}
		}

		String resultStr = str.substring(0, i);
		if (i < str.length()) {
			resultStr += endStr;
		}

		return resultStr;
	}
}
