/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.model.commcode;

import java.util.List;

import kr.sohalee.hangastudio.core.cache.CacheManager;
import kr.sohalee.hangastudio.core.cache.CacheMap;

/**
 * 공통 코드 캐쉬 맵
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public class CommCodeCacheMap extends CacheMap {

	/**
	 * 시스템 함수이므로 호출하지 말것
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void initCache() throws Exception {

		List<CommCodeGroupBean> commCodeGroupList = getSqlMapClient()
				.queryForList(
						"commcode.selectCommCodeGroupListWithCommCodeList",
						null);
		for (CommCodeGroupBean groupBean : commCodeGroupList) {

			groupBean.setCommCodeValueMap(CommCodeUtil
					.getCommCodeValueMap(groupBean.getCommCodeList()));
			put(groupBean.getCommGroupCode(), groupBean);
		}
	}

	private static CommCodeGroupBean getCommCodeGroup(String commGroupCode)
			throws Exception {
		return (CommCodeGroupBean) CacheManager.getInstance().get(
				CommCodeCacheMap.class.getName(), commGroupCode);
	}

	/**
	 * 공통 코드 목록
	 * 
	 * @param commGroupCode
	 *            가져올 코드들의 그룹 코드
	 * @return
	 * @throws Exception
	 */
	public static List<CommCodeBean> getCommCodeList(String commGroupCode)
			throws Exception {

		CommCodeGroupBean groupBean = getCommCodeGroup(commGroupCode);

		return groupBean != null ? groupBean.getCommCodeList() : null;
	}

	/**
	 * 공통 코드 값
	 * 
	 * @return
	 * @throws Exception
	 */
	@SuppressWarnings("unchecked")
	public static String getCommCodeValue(String commGroupCode, String commCode)
			throws Exception {
		return getCommCodeGroup(commGroupCode).getCommCodeValueMap().get(
				commCode);
	}
}
