/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.model.commcode;

import java.util.List;

/**
 * 공통 코드 DAO interface
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public interface CommCodeDao {

	/**
	 * 공통 코드 그룹 상세 (단일)
	 * 
	 * @param commCodeGroupBean
	 *            키
	 * @return
	 * @throws Exception
	 */
	public abstract CommCodeGroupBean getCommCodeGroup(
			CommCodeGroupBean commCodeGroupBean) throws Exception;

	/**
	 * 공통 코드 그룹 목록.
	 * 
	 * @param commCodeGroupBean
	 *            검색 조건
	 * @return
	 * @throws Exception
	 */
	public abstract List<CommCodeGroupBean> getCommCodeGroupList(
			CommCodeGroupBean commCodeGroupBean) throws Exception;

	/**
	 * 공통 코드 그룹 목록. 그룹에 해당하는 코드들의 목록도 함께 가져온다.
	 * 
	 * @param commCodeGroupBean
	 *            검색 조건
	 * @return
	 * @throws Exception
	 */
	public abstract List<CommCodeGroupBean> getCommCodeGroupListWithCommCodeList(
			CommCodeGroupBean commCodeGroupBean) throws Exception;

	/**
	 * 코드 그룹 등록
	 * 
	 * @param commCodeGroupBean
	 *            등록 내용
	 * @throws Exception
	 */
	public abstract void insertCommCodeGroup(CommCodeGroupBean commCodeGroupBean)
			throws Exception;

	/**
	 * 코드 그룹 수정
	 * 
	 * @param commCodeGroupBean
	 *            수정 내용 및 키
	 * @return
	 * @throws Exception
	 */
	public abstract int updateCommCodeGroup(CommCodeGroupBean commCodeGroupBean)
			throws Exception;

	/**
	 * 코드 그룹 삭제. 해당 그룹의 코드들도 함께 삭제
	 * 
	 * @param commCodeGroupBean
	 *            키
	 * @return
	 * @throws Exception
	 */
	public abstract int deleteCommCodeGroup(CommCodeGroupBean commCodeGroupBean)
			throws Exception;

	/**
	 * 코드 상세 보기
	 * 
	 * @param commCodeBean
	 *            키
	 * @return
	 * @throws Exception
	 */
	public abstract CommCodeBean getCommCode(CommCodeBean codeBean)
			throws Exception;

	/**
	 * 코드 목록
	 * 
	 * @param commCodeBean
	 *            검색 조건 (groupCode는 필수)
	 * @return
	 * @throws Exception
	 */
	public abstract List<CommCodeBean> getCommCodeList(CommCodeBean codeBean)
			throws Exception;

	/**
	 * 코드 등록
	 * 
	 * @param commCodeBean
	 *            등록 내용
	 * @throws Exception
	 */
	public abstract void insertCommCode(CommCodeBean commCodeBean)
			throws Exception;

	/**
	 * 코드 수정
	 * 
	 * @param commCodeBean
	 *            수정 내용 및 키
	 * @return
	 * @throws Exception
	 */
	public abstract int updateCommCode(CommCodeBean commCodeBean)
			throws Exception;

	/**
	 * 코드 삭제
	 * 
	 * @param commCodeBean
	 *            키
	 * @return
	 * @throws Exception
	 */
	public abstract int deleteCommCode(CommCodeBean commCodeBean)
			throws Exception;

}
