/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.model.commcode;

import java.util.List;

import kr.sohalee.hangastudio.core.exception.BizException;

/**
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public class CommCodeServiceImpl implements CommCodeService {

	private CommCodeDao commCodeDao = null;

	public void setCommCodeDao(CommCodeDao codeDao) {
		this.commCodeDao = codeDao;
	}

	public CommCodeGroupBean getCommCodeGroup(
			CommCodeGroupBean commCodeGroupBean) throws Exception {

		CommCodeGroupBean groupBean = commCodeDao
				.getCommCodeGroup(commCodeGroupBean);
		if (groupBean == null) {
			throw new BizException("error.get.notFound");
		}

		return groupBean;
	}

	public List<CommCodeGroupBean> getCommCodeGroupList(
			CommCodeGroupBean commCodeGroupBean, boolean withCommCodeList)
			throws Exception {

		if (withCommCodeList) {
			return commCodeDao
					.getCommCodeGroupListWithCommCodeList(commCodeGroupBean);
		} else {
			return commCodeDao.getCommCodeGroupList(commCodeGroupBean);
		}
	}

	public void insertCommCodeGroup(CommCodeGroupBean commCodeGroupBean)
			throws Exception {
		commCodeDao.insertCommCodeGroup(commCodeGroupBean);
	}

	public void updateCommCodeGroup(CommCodeGroupBean commCodeGroupBean)
			throws Exception {
		int affected = commCodeDao.updateCommCodeGroup(commCodeGroupBean);
		if (affected == 0) {
			throw new BizException("error.update.zero");
		} else if (affected > 1) {
			throw new BizException("error.update.over");
		}
	}

	public void deleteCommCodeGroup(CommCodeGroupBean commCodeGroupBean)
			throws Exception {
		int affected = commCodeDao.deleteCommCodeGroup(commCodeGroupBean);
		if (affected == 0) {
			throw new BizException("error.delete.zero");
		} else if (affected > 1) {
			throw new BizException("error.delete.over");
		}
	}

	public CommCodeBean getCommCode(CommCodeBean commCodeBean) throws Exception {

		CommCodeBean codeBean = commCodeDao.getCommCode(commCodeBean);
		if (codeBean == null) {
			throw new BizException("error.get.notFound");
		}

		return codeBean;
	}

	public List<CommCodeBean> getCommCodeList(CommCodeBean commCodeBean)
			throws Exception {
		return commCodeDao.getCommCodeList(commCodeBean);
	}

	public void insertCommCode(CommCodeBean commCodeBean) throws Exception {
		commCodeDao.insertCommCode(commCodeBean);
	}

	public void updateCommCode(CommCodeBean commCodeBean) throws Exception {
		int affected = commCodeDao.updateCommCode(commCodeBean);
		if (affected == 0) {
			throw new BizException("error.update.zero");
		} else if (affected > 1) {
			throw new BizException("error.update.over");
		}
	}

	public void deleteCommCode(CommCodeBean commCodeBean) throws Exception {
		int affected = commCodeDao.deleteCommCode(commCodeBean);
		if (affected == 0) {
			throw new BizException("error.delete.zero");
		} else if (affected > 1) {
			throw new BizException("error.delete.over");
		}
	}

}
