/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.model.mail;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import kr.sohalee.hangastudio.constants.MailConstants;
import kr.sohalee.hangastudio.core.cache.CacheManager;
import kr.sohalee.hangastudio.core.cache.CacheMap;
import kr.sohalee.hangastudio.core.type.KeyValue;
import kr.sohalee.hangastudio.core.util.JavaMailUtil;

/**
 * 메일 캐쉬 맵. 메일에 관련된 코드등을 캐쉬에 넣어 관리한다.
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public class MailCacheMap extends CacheMap {

	/**
	 * 시스템 함수이므로 호출하지 말것
	 */
	@Override
	public void initCache() throws Exception {
		initSysFolderDispName();
		initContentType();
	}

	private void initSysFolderDispName() throws Exception {

		HashMap<String, String> map = new HashMap<String, String>();
		map.put(MailConstants.SYS_FOLDER_ABS_INBOX, "받은편지함");
		map.put(MailConstants.SYS_FOLDER_ABS_OUTBOX, "보낸편지함");
		map.put(MailConstants.SYS_FOLDER_ABS_TRASH, "지운편지함");

		put(MailConstants.CACHE_KEY_SYS_FOLDER_DISP_NAME, map);
	}

	private void initContentType() throws Exception {

		List<KeyValue> list = new ArrayList<KeyValue>();
		list.add(new KeyValue(JavaMailUtil.MULTIPART_ALTERNATIVE, "혼합(권장)"));
		list.add(new KeyValue(JavaMailUtil.TEXT_PLAIN, "Text"));
		list.add(new KeyValue(JavaMailUtil.TEXT_HTML, "Html"));

		put(MailConstants.CACHE_KEY_CONTENT_TYPE, list);
	}

	/**
	 * 메일 발송등의 화면에서 사용하는 Content Type.
	 * 
	 * @return
	 * @throws Exception
	 */
	@SuppressWarnings("unchecked")
	public static Collection<KeyValue> getContentTypes() throws Exception {
		return (Collection) CacheManager.getInstance().get(
				MailCacheMap.class.getName(),
				MailConstants.CACHE_KEY_CONTENT_TYPE);
	}

	/**
	 * 시스템 폴더 화면 표시 명.
	 * 
	 * @return
	 * @throws Exception
	 */
	@SuppressWarnings("unchecked")
	public static Map<String, String> getSysFolderDispName() throws Exception {
		return (Map) CacheManager.getInstance().get(
				MailCacheMap.class.getName(),
				MailConstants.CACHE_KEY_SYS_FOLDER_DISP_NAME);
	}
}
