/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.model.mail;

import java.util.Properties;

import javax.mail.AuthenticationFailedException;
import javax.mail.Folder;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Store;

import kr.sohalee.hangastudio.constants.MailConstants;
import kr.sohalee.hangastudio.core.exception.BizConnectException;
import kr.sohalee.hangastudio.core.exception.KnownException;
import kr.sohalee.hangastudio.core.system.SSLInitializer;
import kr.sohalee.hangastudio.core.util.BooleanUtil;
import kr.sohalee.hangastudio.core.util.JavaMailUtil;
import kr.sohalee.hangastudio.model.mail.acc.AccRecvBean;

import org.apache.log4j.Logger;

/**
 * JavaMail의 Store를 관리하는 Util Bean
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public class MailStoreBean {

	private static Logger logger = Logger.getLogger(MailStoreBean.class);

	private Store store = null;

	public MailStoreBean(final AccRecvBean accRecvBean) throws Exception {

		String protocol = accRecvBean.getProtocol();

		Properties props = new Properties();
		props.put("mail." + protocol + ".host", accRecvBean.getHost());
		props.put("mail." + protocol + ".port", accRecvBean.getPort());

		// SSL 처리
		if (BooleanUtil.booleanValue(accRecvBean.getSsl())) {
			SSLInitializer.init();
			JavaMailUtil.setPropertiesForSsl(props, protocol, accRecvBean
					.getSslPort());
		}

		Session session = Session.getInstance(props, null);

		store = session.getStore(protocol);

		try {
			store.connect(accRecvBean.getId(), accRecvBean.getPassword());
		} catch (AuthenticationFailedException ae) {
			throw new BizConnectException(MailConstants.RES_KEY_ERR_RECV_AUTH,
					ae);
		} catch (MessagingException me) {
			throw new BizConnectException(
					MailConstants.RES_KEY_ERR_RECV_CONNECT, me);
		}

		if (logger.isDebugEnabled()) {
			logger.debug("store Connection / session properties :"
					+ session.getProperties().toString());
		}

		props = null;
		session = null;
	}

	public MailStoreBean close() throws Exception {

		if (store != null && store.isConnected()) {
			store.close();
		}

		return null;
	}

	public Store getStore() {
		return store;
	}

	public Folder getDefaultFolder() throws Exception {

		Folder folder = store.getDefaultFolder();
		if (folder == null) {
			throw new KnownException("default folder not found");
		}

		return folder;
	}

	public Folder getOpenedFolder(String name, int mode) throws Exception {

		Folder folder = getDefaultFolder();
		folder = folder.getFolder(name);
		if (folder == null) {
			throw new KnownException("'" + name + "' folder not found");
		}

		folder.open(mode);

		return folder;
	}
}
