/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.model.mail.acc;

import java.util.List;

import kr.sohalee.hangastudio.constants.ConfigConstants;
import kr.sohalee.hangastudio.core.exception.BizException;
import kr.sohalee.hangastudio.core.exception.BizInputException;
import kr.sohalee.hangastudio.core.util.BooleanUtil;
import kr.sohalee.hangastudio.model.config.ConfigCacheMap;

/**
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public class AccServiceImpl implements AccService {

	private AccDao accDao = null;

	public void setAccDao(AccDao accDao) {
		this.accDao = accDao;
	}

	private void changeAccRecvBeanForSystem(AccRecvBean targetBean)
			throws Exception {

		targetBean.setAccRecvName(ConfigCacheMap
				.getConfigValue(ConfigConstants.CODE_MAIL_RECV_DISPLAY_NAME));
		targetBean.setAddressHost(ConfigCacheMap
				.getConfigValue(ConfigConstants.CODE_MAIL_RECV_ADDRESS_HOST));
		targetBean.setProtocol(ConfigCacheMap
				.getConfigValue(ConfigConstants.CODE_MAIL_RECV_PROTOCOL));
		targetBean.setHost(ConfigCacheMap
				.getConfigValue(ConfigConstants.CODE_MAIL_RECV_HOST));
		targetBean.setPort(ConfigCacheMap
				.getConfigValue(ConfigConstants.CODE_MAIL_RECV_PORT));
		targetBean.setSsl(Boolean.valueOf(ConfigCacheMap
				.getConfigValue(ConfigConstants.CODE_MAIL_RECV_SSL)));
		targetBean.setSslPort(ConfigCacheMap
				.getConfigValue(ConfigConstants.CODE_MAIL_RECV_SSL_PORT));
		targetBean.setInbox(ConfigCacheMap
				.getConfigValue(ConfigConstants.CODE_MAIL_INBOX));
		targetBean.setOutbox(ConfigCacheMap
				.getConfigValue(ConfigConstants.CODE_MAIL_OUTBOX));
		targetBean.setTrash(ConfigCacheMap
				.getConfigValue(ConfigConstants.CODE_MAIL_TRASH));
	}

	public AccRecvBean getAccRecv(AccRecvBean accRecvBean) throws Exception {

		AccRecvBean recvBean = accDao.getAccRecv(accRecvBean);
		if (recvBean == null) {
			throw new BizException("error.get.notFound");
		} else if (recvBean.getSystem().booleanValue()) {
			changeAccRecvBeanForSystem(recvBean);
		}

		return recvBean;
	}

	public List<AccRecvBean> getAccRecvList(AccRecvBean accRecvBean)
			throws Exception {

		boolean systemUse = Boolean.valueOf(ConfigCacheMap
				.getConfigValue(ConfigConstants.CODE_MAIL_RECV_USE));

		List<AccRecvBean> accRecvList = accDao.getAccRecvList(accRecvBean);
		for (int i = accRecvList.size() - 1; i >= 0; i--) {

			AccRecvBean recvBean = accRecvList.get(i);

			if (BooleanUtil.booleanValue(recvBean.getSystem())) {

				if (!systemUse) {
					accRecvList.remove(i);
					continue;
				}

				changeAccRecvBeanForSystem(recvBean);
			}
		}

		return accRecvList;
	}

	public Integer insertAccRecv(AccRecvBean accRecvBean) throws Exception {

		if (accRecvBean.getDefault().booleanValue()) {
			accDao.updateAccRecvCancelDefault(accRecvBean);
		}

		return accDao.insertAccRecv(accRecvBean);
	}

	public void updateAccRecv(AccRecvBean accRecvBean) throws Exception {

		// 기본 계정 사용 여부에 따른 기존의 기본 계정 해제
		AccRecvBean recvBean = accDao.getAccRecv(accRecvBean);
		if (recvBean.getDefault().booleanValue()) {

			// Default는 false에서 true로만 변환되도록 만듬. 받드시 하나는 Default여야 하므로
			// false인 경우 시스템 계정을 기본으로 해도 되겠으나 시스템이 없을수도 있고해서 여러가지를 확인해야함
			if (!accRecvBean.getDefault().booleanValue()) {
				throw new BizInputException(
						"error.mail.acc.recv.canNotCancelDefault");
			}
		} else {

			if (accRecvBean.getDefault().booleanValue()) {
				accDao.updateAccRecvCancelDefault(accRecvBean);
			}
		}

		int affected = accDao.updateAccRecv(accRecvBean);
		if (affected == 0) {
			throw new BizException("error.update.zero");
		} else if (affected > 1) {
			throw new BizException("error.update.over");
		}
	}

	public void deleteAccRecv(AccRecvBean accRecvBean) throws Exception {

		AccRecvBean recvBean = accDao.getAccRecv(accRecvBean);
		if (recvBean.getSystem().booleanValue()) {
			throw new BizException("error.mail.acc.recv.canNotDeleteSystem");
		}
		if (accRecvBean.getDefault().booleanValue()) {
			throw new BizException("error.mail.acc.recv.canNotDeleteDefault");
		}

		int affected = accDao.deleteAccRecv(accRecvBean);
		if (affected == 0) {
			throw new BizException("error.delete.zero");
		} else if (affected > 1) {
			throw new BizException("error.delete.over");
		}
	}

}
