/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.model.mail.folder;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.mail.Folder;

import kr.sohalee.hangastudio.core.util.JavaMailUtil;
import kr.sohalee.hangastudio.model.mail.MailStoreBean;
import kr.sohalee.hangastudio.model.mail.acc.AccRecvBean;

/**
 * 메일 폴더 DAO 구상 클래스 (JavaMail)
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public class FolderDaoImplJavaMail implements FolderDao {

	private FolderBean getFolder(Folder folder) throws Exception {

		try {

			// JavaMail 폴더를 READ_ONLY로 연다
			folder.open(Folder.READ_ONLY);

			// JavaMail 폴더를 FolderBean에 넣는다.
			FolderBean folderBean = new FolderBean();
			folderBean.setName(folder.getName());
			folderBean.setMessageCount(Integer
					.valueOf(folder.getMessageCount()));

			return folderBean;
		} finally {
			JavaMailUtil.closeFolder(folder);
		}
	}

	private List<FolderBean> getFolderListFlat(Folder folder) throws Exception {

		List<FolderBean> folderList = null;

		// JavaMail 폴더목록을 가져온다.
		Folder folderArray[] = folder.list("*");
		if (folderArray != null && folderArray.length > 0) {

			// JavaMail 폴더목록을 FolderBean List에 담는다.
			folderList = new ArrayList<FolderBean>();
			for (Folder subFolder : folderArray) {

				FolderBean folderBean = getFolder(subFolder);
				folderList.add(folderBean);

				folderBean = null;
			}
		}

		folderArray = null;

		return folderList;
	}

	private List<FolderBean> getFolderListTree(Folder folder) throws Exception {

		List<FolderBean> folderList = null;

		// JavaMail 폴더목록을 가져온다.
		Folder folderArray[] = folder.list();
		if (folderArray != null && folderArray.length > 0) {

			// JavaMail 폴더목록을 FolderBean List에 담는다.
			folderList = new ArrayList<FolderBean>();
			for (Folder subFolder : folderArray) {

				FolderBean folderBean = getFolder(subFolder);
				folderList.add(folderBean);
				folderBean.setChildFolderList(getFolderListTree(subFolder));

				folderBean = null;
			}
		}

		folderArray = null;

		return folderList;
	}

	private List<FolderBean> getFolderList(Folder folder, boolean flat)
			throws Exception {

		if (flat) {
			return getFolderListFlat(folder);
		} else {
			return getFolderListTree(folder);
		}
	}

	public FolderBean getFolder(AccRecvBean accRecvBean, FolderBean folderBean)
			throws Exception {

		// JavaMail Store를 래핑한 Bean을 생성한다.
		MailStoreBean storeBean = new MailStoreBean(accRecvBean);

		try {

			FolderBean resultFolderBean = null;

			Folder folder = storeBean.getDefaultFolder();
			// JavaMail Folder를 가져온다. 폴더명은 name
			folder = folder.getFolder(folderBean.getName());
			if (folder != null) {
				resultFolderBean = getFolder(folder);
			}

			folder = null;

			return resultFolderBean;
		} finally {
			storeBean = storeBean.close();
		}
	}

	public List<FolderBean> getFolderList(AccRecvBean accRecvBean,
			boolean flat, boolean withoutSysFolder) throws Exception {

		// JavaMail Store를 래핑한 Bean을 생성한다.
		MailStoreBean storeBean = new MailStoreBean(accRecvBean);

		try {

			// JavaMail 최상위 폴더를 가져온다.
			Folder folder = storeBean.getDefaultFolder();

			// JavaMail 폴더목록을 FolderBean List에 담는다.
			List<FolderBean> folderList = getFolderList(folder, flat);
			// 시스템 폴더는 반환할 list에서 제거한다.
			if (withoutSysFolder && folderList != null) {

				Pattern pattern = Pattern.compile(accRecvBean
						.getMailboxPattern(), Pattern.CASE_INSENSITIVE);

				for (int i = folderList.size() - 1; i >= 0; i--) {

					Matcher matcher = pattern.matcher(folderList.get(i)
							.getName());
					if (matcher.matches()) {
						folderList.remove(i);
					}
				}
			}

			folder = null;

			return folderList;
		} finally {
			storeBean = storeBean.close();
		}
	}

}
