/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.model.mail.folder;

import java.util.Map;

import kr.sohalee.hangastudio.constants.MailConstants;
import kr.sohalee.hangastudio.model.mail.MailCacheMap;
import kr.sohalee.hangastudio.model.mail.acc.AccRecvBean;
import kr.sohalee.hangastudio.presentation.service.mail.MailSupport;

import org.apache.log4j.Logger;

import com.opensymphony.xwork2.ActionContext;

/**
 * 폴더 관련 Util
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public class FolderUtil {

	private static Logger logger = Logger.getLogger(FolderUtil.class);

	/**
	 * 폴더의 화면 표시용 이름을 반환한다. 주로 시스템 폴더가 해당된다.
	 * 
	 * @param folderName
	 *            폴더 명
	 * @return
	 * @throws Exception
	 */
	public static String getDisplayName(String folderName) throws Exception {

		try {

			MailSupport action = (MailSupport) ActionContext.getContext()
					.getActionInvocation().getAction();
			AccRecvBean accRecvBean = action.getSessAccRecvBean();

			Map<String, String> sysFolderMap = MailCacheMap
					.getSysFolderDispName();

			String displayName = folderName;

			if (folderName.equalsIgnoreCase(accRecvBean.getInbox())) {
				displayName = sysFolderMap
						.get(MailConstants.SYS_FOLDER_ABS_INBOX);
			} else if (folderName.equalsIgnoreCase(accRecvBean.getOutbox())) {
				displayName = sysFolderMap
						.get(MailConstants.SYS_FOLDER_ABS_OUTBOX);
			} else if (folderName.equalsIgnoreCase(accRecvBean.getTrash())) {
				displayName = sysFolderMap
						.get(MailConstants.SYS_FOLDER_ABS_TRASH);
			}

			return displayName;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw e;
		}
	}
}
