/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.model.mail.message;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import kr.sohalee.hangastudio.core.bean.PagingBaseBean;
import kr.sohalee.hangastudio.core.util.DateUtil;

/**
 * 메일 메세지 Bean
 * 
 * @author hangaroy
 * @version $Id$
 */
@SuppressWarnings("serial")
public class MessageBean extends PagingBaseBean {

	/** 폴더 명 */
	private String folderName = null;

	/** 메세지 번호 */
	private Integer no = null;

	/** 잠금 여부 */
	private Boolean lock = null;

	/** 읽은 편지인지 여부 */
	private Boolean read = null;

	/** 메세지 Encoding */
	private String encoding = null;

	/** 보낸이 이름 */
	private String fromName = null;

	/** 보낸이 이메일 주소 */
	private String fromAddress = null;

	/** 받는이 이메일 주소 */
	private String to = null;

	/** 참조 받는이 이메일 주소 */
	private String cc = null;

	/** 숨은 참조 받는이 이메일 주소 */
	private String bcc = null;

	/** 제목 */
	private String subject = null;

	/** 본문 형식 (text/plain or text/html...) */
	private String contentMimeType = null;

	/** 본문내용 */
	private String content = null;

	/** 날짜와 시간 */
	private Date date = null;

	/** 용량 */
	private Integer size = Integer.valueOf(0);

	/** 보낸편지함에 저장 */
	private Boolean save = null;

	/** 이동시 폴더명, 메일 발송시 보낸편지함 */
	private String saveFolderName = null;

	/** 전달 여부 */
	private Boolean forward = null;

	/** 첨부파일 목록 */
	private List<MessageAttachBean> attachList = null;

	/** 자식 메세지 목록 */
	private List<MessageBean> childMessageList = null;

	public void addAttachList(MessageAttachBean attachBean) {

		if (attachList == null) {
			attachList = new ArrayList<MessageAttachBean>();
		}

		attachList.add(attachBean);
	}

	public void addChildMessageList(MessageBean messageBean) {

		if (childMessageList == null) {
			childMessageList = new ArrayList<MessageBean>();
		}

		childMessageList.add(messageBean);
	}

	public String getDateStr() {
		return DateUtil.getString(date, "yyyyMMdd");
	}

	public void setDateStr(String date) throws Exception {
		this.date = DateUtil.getDate(date, "yyyyMMdd");
	}

	public String getFolderName() {
		return folderName;
	}

	public String getFolderNameUrl() throws Exception {
		return getUrl(folderName);
	}

	public void setFolderName(String folderName) {
		this.folderName = folderName;
	}

	public Integer getNo() {
		return no;
	}

	public void setNo(Integer no) {
		this.no = no;
	}

	public Boolean getLock() {
		return lock;
	}

	public void setLock(Boolean lock) {
		this.lock = lock;
	}

	public Boolean getRead() {
		return read;
	}

	public void setRead(Boolean read) {
		this.read = read;
	}

	public String getEncoding() {
		return encoding;
	}

	public void setEncoding(String encoding) {
		this.encoding = encoding;
	}

	public String getFromName() {
		return fromName;
	}

	public String getFromNameUrl() throws Exception {
		return getUrl(fromName);
	}

	public void setFromName(String fromName) {
		this.fromName = fromName;
	}

	public String getFromAddress() {
		return fromAddress;
	}

	public void setFromAddress(String fromAddress) {
		this.fromAddress = fromAddress;
	}

	public String getTo() {
		return to;
	}

	public void setTo(String to) {
		this.to = to;
	}

	public String getCc() {
		return cc;
	}

	public void setCc(String cc) {
		this.cc = cc;
	}

	public String getBcc() {
		return bcc;
	}

	public void setBcc(String bcc) {
		this.bcc = bcc;
	}

	public String getSubject() {
		return subject;
	}

	public String getSubjectUrl() throws Exception {
		return getUrl(subject);
	}

	public void setSubject(String subject) {
		this.subject = subject;
	}

	public String getContentMimeType() {
		return contentMimeType;
	}

	public void setContentMimeType(String contentMimeType) {
		this.contentMimeType = contentMimeType;
	}

	public String getContent() {
		return content;
	}

	public void setContent(String content) {
		this.content = content;
	}

	public Date getDate() {
		return date;
	}

	public void setDate(Date date) {
		this.date = date;
	}

	public Integer getSize() {
		return size;
	}

	public void setSize(Integer size) {
		this.size = size;
	}

	public Boolean getSave() {
		return save;
	}

	public void setSave(Boolean save) {
		this.save = save;
	}

	public String getSaveFolderName() {
		return saveFolderName;
	}

	public void setSaveFolderName(String saveFolderName) {
		this.saveFolderName = saveFolderName;
	}

	public Boolean getForward() {
		return forward;
	}

	public void setForward(Boolean forward) {
		this.forward = forward;
	}

	public List<MessageAttachBean> getAttachList() {
		return attachList;
	}

	public void setAttachList(List<MessageAttachBean> attachList) {
		this.attachList = attachList;
	}

	public List<MessageBean> getChildMessageList() {
		return childMessageList;
	}

	public void setChildMessageList(List<MessageBean> childMessageList) {
		this.childMessageList = childMessageList;
	}

}