/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.model.mail.message;

import java.util.List;

import kr.sohalee.hangastudio.model.mail.acc.AccRecvBean;
import kr.sohalee.hangastudio.model.mail.acc.AccSendBean;

/**
 * 메일 메세지 Service interface<br />
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public interface MessageService {

	/**
	 * 메일 메세지 상세
	 * 
	 * @param accRecvBean
	 *            메세지가 존재하는 폴더
	 * @param messageBean
	 *            폴더명 및 키
	 * @return
	 * @throws Exception
	 */
	public abstract MessageBean getMessage(AccRecvBean accRecvBean,
			MessageBean messageBean) throws Exception;

	/**
	 * 폴더내의 메일 메세지 목록
	 * 
	 * @param accRecvBean
	 *            메세지들이 존재하는 폴더
	 * @param messageBean
	 *            폴더명 및 검색 조건
	 * @return
	 * @throws Exception
	 */
	public abstract List<MessageBean> getMessageList(AccRecvBean accRecvBean,
			MessageBean messageBean) throws Exception;

	/**
	 * 폴더내의 메일 메세지 수
	 * 
	 * @param accRecvBean
	 *            받는 서버 계정 정보
	 * @param messageBean
	 *            폴더명 및 검색 조건
	 * @return
	 * @throws Exception
	 */
	public abstract Integer getMessageCount(AccRecvBean accRecvBean,
			MessageBean messageBean) throws Exception;

	/**
	 * 메일 메세지 전송
	 * 
	 * @param accSendBean
	 *            보내는 서버 계정 정보
	 * @param accRecvBean
	 *            받는 서버 계정 정보
	 * @param messageBean
	 *            폴더명 및 메세지 내용
	 * @throws Exception
	 */
	public abstract void insertMessage(AccSendBean accSendBean,
			AccRecvBean accRecvBean, MessageBean messageBean) throws Exception;

	/**
	 * 메일 메세지 플래그 변경. messageBean에 설정된 값에 의해 각종 플래그를 변경
	 * 
	 * @param accRecvBean
	 *            받는 서버 계정 정보
	 * @param messageBean
	 *            폴더명 및 키 와 플래그
	 * @throws Exception
	 */
	public abstract void updateMessageFlag(AccRecvBean accRecvBean,
			MessageBean messageBean) throws Exception;

	/**
	 * 메일 메세지 읽음 플래그 변경 (읽음 하나만 변경)<br />
	 * updateMessageFlag를 사용해도 되지만 읽기는 빈번하게 발생하므로 불필요한 부분을 줄인다.
	 * 
	 * @param accRecvBean
	 *            받는 서버 계정 정보
	 * @param messageBean
	 *            키 및 읽음 플래그
	 * @throws Exception
	 */
	public abstract void updateMessageReadFlag(AccRecvBean accRecvBean,
			MessageBean messageBean) throws Exception;

	/**
	 * 메일 메세지들 삭제
	 * 
	 * @param accRecvBean
	 *            받는 서버 계정 정보
	 * @param messageBean
	 *            폴더명 및 삭제할 메세지 목록. childMessageList에 담아 보내야한다.
	 * @throws Exception
	 */
	public abstract void deleteMessageList(AccRecvBean accRecvBean,
			MessageBean messageBean) throws Exception;

	/**
	 * 메일 메세지들 이동
	 * 
	 * @param accRecvBean
	 *            받는 서버 계정 정보
	 * @param messageBean
	 *            폴더명 및 이동할 메세지 목록. childMessageList에 담아 보내야한다.
	 * @throws Exception
	 */
	public abstract void moveMessageList(AccRecvBean accRecvBean,
			MessageBean messageBean) throws Exception;

	/**
	 * 메일 메세지의 첨부파일 상세
	 * 
	 * @param accRecvBean
	 *            받는 서버 계정
	 * @param messageBean
	 *            폴더명 및 메세지 키
	 * @param attachBean
	 *            첨부파일의 키 또는 경로
	 * @return
	 * @throws Exception
	 */
	public abstract MessageAttachBean getMessageAttach(AccRecvBean accRecvBean,
			MessageBean messageBean, MessageAttachBean attachBean)
			throws Exception;

}
