/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.model.mail.message;

import kr.sohalee.hangastudio.core.bean.PagingBaseBean;
import kr.sohalee.hangastudio.core.util.HtmlUtil;
import kr.sohalee.hangastudio.core.util.JavaMailUtil;

import org.apache.log4j.Logger;

/**
 * 메일 메세지 관련 Util
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
public class MessageUtil {

	@SuppressWarnings("unused")
	private static Logger logger = Logger.getLogger(MessageUtil.class);

	public static void setPagingValue(PagingBaseBean pagingBean, int pageNo,
			int listScale, int count) {

		int start = (count - (pageNo * listScale)) + 1;

		pagingBean.setStartNo(Integer.valueOf(start));
		pagingBean.setEndNo(Integer.valueOf((start + listScale) - 1));

		if (pagingBean.getStartNo() < 1) {
			pagingBean.setStartNo(1);
		}
	}

	public static String getContentForHtml(MessageBean messageBean) {

		String result = messageBean.getContent();

		if (messageBean.getContentMimeType().trim().equalsIgnoreCase(
				JavaMailUtil.TEXT_HTML)) {

			result = HtmlUtil.removeTag(result, "html", false);
			result = HtmlUtil.removeTag(result, "head", true);
			result = HtmlUtil.removeTag(result, "meta", false);
			result = HtmlUtil.removeTag(result, "link", true);
			result = HtmlUtil.removeTag(result, "style", true);
			result = HtmlUtil.removeTag(result, "script", true);
			result = HtmlUtil.removeTag(result, "body", false);
		} else {
			result = HtmlUtil.toText(result, true);
		}

		return result;
	}

//	private static String getAttachUrlInit() throws Exception {
//
//		HttpServletRequest request = ServletActionContext.getRequest();
//
//		/*
//		 * 기본정보
//		 */
//		String contextPath = ServletActionContext.getRequest().getContextPath();
//		String namespace = ServletActionContext.getActionMapping()
//				.getNamespace();
//
//		/*
//		 * Url에 넣을 파라미터
//		 */
//		String folderName = (String) request
//				.getAttribute("messageBean.folderName");
//		Integer messageNo = (Integer) request.getAttribute("messageBean.no");
//
//		StringBuffer sBuf = new StringBuffer();
//
//		sBuf.append(contextPath);
//		sBuf.append(namespace);
//		sBuf.append("/messageAttachDownload." + Constants.STRUTS_EXTENSION);
//		sBuf.append("?messageBean.folderName="
//				+ URLEncoder.encode(folderName, Constants.APP_ENCODING));
//		sBuf.append("&messageBean.no=" + String.valueOf(messageNo.intValue()));
//
//		folderName = null;
//		messageNo = null;
//		contextPath = null;
//		namespace = null;
//		request = null;
//
//		return sBuf.toString();
//	}
//
//	/**
//	 * 메세지 항목의 Html Tag를 반환한다. (기본형태의 항목)
//	 * 
//	 * @param title
//	 *            왼쪽에 표시되는 항목명
//	 * @param text
//	 *            오른쪽에 표시되는 내용
//	 * @return String Html Tag
//	 * @throws Exception
//	 */
//	private static String getChildMessageItemTag(String itemName, String text)
//			throws Exception {
//
//		StringBuffer sBuf = new StringBuffer();
//
//		sBuf.append("	<tr>\n");
//		sBuf.append("		<th>\n");
//		sBuf.append(itemName);
//		sBuf.append("		</th>\n");
//		sBuf.append("		<td>\n");
//		sBuf.append(text);
//		sBuf.append("		</td>\n");
//		sBuf.append("	</tr>\n");
//
//		return sBuf.toString();
//	}
//
//	public static String getChildMessageForView(List<MessageBean> messageList)
//			throws Exception {
//
//		try {
//
//			String attUrlInit = getAttachUrlInit();
//			ActionSupport action = (ActionSupport) ActionContext.getContext()
//					.getActionInvocation().getAction();
//
//			StringBuffer sBuf = new StringBuffer();
//
//			for (MessageBean bean : messageList) {
//
//				sBuf
//						.append("<p style=\"padding: 15px; padding-top: 30px;\" />\n");
//				sBuf.append("<table style=\"width: 100%;\">\n");
//
//				/* 보낸이 */
//				String from = "";
//				if (bean.getFromName() != null) {
//					from = bean.getFromName() + " ";
//				}
//				from += "&lt;" + bean.getFromAddress() + "&gt;";
//				sBuf.append(getChildMessageItemTag(action
//						.getText("mail.message.label.from"), from));
//
//				/* 제목 */
//				sBuf.append(getChildMessageItemTag(action
//						.getText("mail.message.label.subject"), bean
//						.getSubject()));
//
//				/* 보낸날짜 */
//				String date = DateUtil.getString(bean.getDate(),
//						"yyyy/MM/dd HH:mm:ss E z Z");
//				sBuf.append(getChildMessageItemTag(action
//						.getText("mail.message.label.date"), date));
//
//				/*
//				 * 내용 & 서브 메세지 (message/rfc822)
//				 */
//				sBuf.append("	<tr>\n");
//				sBuf.append("		<td colspan=\"2\" style=\"padding: 10px;\">\n");
//				/* 내용 */
//				sBuf.append("			<div class=\"mailContent\">");
//				sBuf.append(getContentForHtml(bean));
//				sBuf.append("			</div>");
//
//				/* 서브 메세지 */
//				if (bean.getChildMessageList() != null) {
//					// 재귀함수 호출.
//					sBuf.append(getChildMessageForView(bean
//							.getChildMessageList()));
//				}
//				sBuf.append("		</td>\n");
//				sBuf.append("	</tr>\n");
//
//				/*
//				 * 첨부파일
//				 */
//				if (!CollectionUtil.isEmpty(bean.getAttachList())) {
//
//					for (MessageAttachBean attachBean : bean.getAttachList()) {
//
//						// 첨부파일 다운로드 Url
//						String attachUrl = attUrlInit
//								+ "&attachBean.multipartPath="
//								+ attachBean.getMultipartPath();
//						// 첨부파일명에 하이퍼링크가 걸린 Html Tag
//						String hrefTag = "<a href=\"" + attachUrl + "\">"
//								+ attachBean.getFileName() + "</a>";
//
//						sBuf
//								.append(getChildMessageItemTag(action
//										.getText("mail.message.label.attach"),
//										hrefTag));
//					}
//				}
//
//				sBuf.append("</table>\n");
//			}
//
//			attUrlInit = null;
//			action = null;
//
//			return sBuf.toString();
//		} catch (Exception e) {
//
//			logger.error(e.getMessage(), e);
//			throw e;
//		}
//	}
}
