/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.model.user;

import java.util.List;

import kr.sohalee.hangastudio.core.exception.BizException;
import kr.sohalee.hangastudio.core.util.StringUtil;
import kr.sohalee.hangastudio.model.mail.acc.AccDao;
import kr.sohalee.hangastudio.model.mail.acc.AccRecvBean;

public class UserServiceImpl implements UserService {

	private UserDao userDao = null;

	private AccDao mailAccDao = null;

	public void setUserDao(UserDao userDao) {
		this.userDao = userDao;
	}

	public void setMailAccDao(AccDao mailAccDao) {
		this.mailAccDao = mailAccDao;
	}

	public UserBean getUser(UserBean userBean) throws Exception {

		UserBean uBean = userDao.getUser(userBean);
		if (uBean == null) {
			throw new BizException("error.user.userNotFound");
		}

		return uBean;
	}

	public List<UserBean> getUserList(UserBean userBean) throws Exception {
		return userDao.getUserList(userBean);
	}

	public Integer getUserCount(UserBean userBean) throws Exception {
		return userDao.getUserCount(userBean);
	}

	public Integer insertUser(UserBean userBean) throws Exception {

		if (userDao.getUser(userBean) != null) {
			throw new BizException("error.user.userIdAlreadyExist");
		}

		Integer insertId = userDao.insertUser(userBean);
		if (insertId != null) {

			AccRecvBean accRecvBean = new AccRecvBean();
			accRecvBean.setDefault(Boolean.valueOf(true));
			accRecvBean.setSystem(Boolean.valueOf(true));
			accRecvBean.setUserId(userBean.getUserId());
			accRecvBean.setId(userBean.getUserId());
			accRecvBean.setPassword(userBean.getPassword());
			mailAccDao.insertAccRecv(accRecvBean);
			
			accRecvBean = null;
		}

		return insertId;
	}

	public void updateUser(UserBean userBean) throws Exception {
		int affected = userDao.updateUser(userBean);
		if (affected == 0) {
			throw new BizException("error.update.zero");
		} else if (affected > 1) {
			throw new BizException("error.update.over");
		}
	}

	public void deleteUser(UserBean userBean) throws Exception {
		int affected = userDao.deleteUser(userBean);
		if (affected == 0) {
			throw new BizException("error.delete.zero");
		} else if (affected > 1) {
			throw new BizException("error.delete.over");
		}
	}

	public UserBean login(UserBean userBean) throws Exception {

		UserBean resultBean = getUser(userBean);

		if (!StringUtil.isEquals(resultBean.getPassword(), userBean
				.getPassword())) {
			throw new BizException("error.user.passwordMismatch");
		}

		return resultBean;
	}

}
