/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.presentation.service.mail.acc;

import java.util.List;

import kr.sohalee.hangastudio.constants.MailConstants;
import kr.sohalee.hangastudio.core.exception.BizInputException;
import kr.sohalee.hangastudio.core.util.BooleanUtil;
import kr.sohalee.hangastudio.core.util.IntegerUtil;
import kr.sohalee.hangastudio.model.mail.acc.AccRecvBean;
import kr.sohalee.hangastudio.presentation.service.mail.MailSupport;

/**
 * 받는 메일 서버 계정 관리 Action
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
@SuppressWarnings("serial")
public class AccAction extends MailSupport {

	/*
	 * View에서 사용하는 Property
	 */

	private AccRecvBean accRecvBean = null;

	private List<AccRecvBean> accRecvList = null;

	public AccRecvBean getAccRecvBean() {
		return accRecvBean;
	}

	public void setAccRecvBean(AccRecvBean accRecvBean) {
		this.accRecvBean = accRecvBean;
	}

	public List<AccRecvBean> getAccRecvList() {
		return accRecvList;
	}

	public void setAccRecvList(List<AccRecvBean> accRecvList) {
		this.accRecvList = accRecvList;
	}

	/*
	 * Action Execute Method
	 */

	/**
	 * 받는 서버 계정 목록
	 * 
	 * @return
	 * @throws Exception
	 */
	public String executeRecvList() throws Exception {

		if (accRecvBean == null) {
			accRecvBean = new AccRecvBean();
		}

		accRecvBean.setUserId(getSessUserBean().getUserId());
		accRecvList = accService.getAccRecvList(accRecvBean);

		return SUCCESS;
	}

	public String executeRecvView() throws Exception {

		if (accRecvBean == null || accRecvBean.getAccRecvSeq() == null) {
			addActionError("error.mail.param.acc.recv.seq");
			return ERROR;
		}

		accRecvBean.setUserId(getSessUserBean().getUserId());
		accRecvBean = accService.getAccRecv(accRecvBean);

		return SUCCESS;
	}

	public String executeRecvInsertForm() throws Exception {
		return SUCCESS;
	}

	public String executeRecvInsertAction() throws Exception {

		// TODO Validation을 넣고 아래를 지우자
		if (accRecvBean == null) {
			addActionError("error.mail.param.acc.recv");
			return ERROR;
		}

		accRecvBean.setUserId(getSessUserBean().getUserId());
		accService.insertAccRecv(accRecvBean);

		/*
		 * 등록시 선택되어있는 계정이 바뀔수 있는 정보는 default 값 뿐이지만 그래도 정보가 바뀌는것이니 세션을 갱신해주자
		 */
		AccRecvBean selectedRecvBean = getSessAccRecvBean();
		removeSessionObject(MailConstants.SESS_ACC_RECV_LIST);
		changeSessAccRecvBean(selectedRecvBean);

		return SUCCESS;
	}

	public String executeRecvUpdateForm() throws Exception {
		return executeRecvView();
	}

	public String executeRecvUpdateAction() throws Exception {

		// TODO Validation을 넣고 아래를 지우자
		if (accRecvBean == null || accRecvBean.getAccRecvSeq() == null) {
			addActionError("error.mail.param.acc.recv");
			return ERROR;
		}

		try {

			accRecvBean.setUserId(getSessUserBean().getUserId());
			accService.updateAccRecv(accRecvBean);

			// 계정목록을 세션에서 삭제
			removeSessionObject(MailConstants.SESS_ACC_RECV_LIST);
			// 현재 선택되어 작동하는 계정과 수정한 계정이 같은 경우 데이타 갱신
			if (IntegerUtil.compareIntValue(getSessAccRecvBean()
					.getAccRecvSeq(), accRecvBean.getAccRecvSeq())) {
				removeSessionObject(MailConstants.SESS_ACC_RECV);
				changeSessAccRecvBean(accRecvBean);
			}
		} catch (BizInputException bie) {

			// BizInputException의 경우 interceptor가 잡아서 INPUT으로 돌려주게 하였으므로 생략하지만
			// 시스템 계정인경우 특별하게 Bean에 값을 다시 넣어야하므로 별도로 처리한다.
			if (BooleanUtil.booleanValue((accRecvBean.getSystem()))) {
				AccRecvBean recvBean = accService.getAccRecv(accRecvBean);
				recvBean.setId(accRecvBean.getId());
				recvBean.setPassword(accRecvBean.getPassword());
				accRecvBean = recvBean;
				recvBean = null;
			}

			throw bie;
		}

		return SUCCESS;
	}

}
