/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.presentation.service.mail.message;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import kr.sohalee.hangastudio.constants.Constants;
import kr.sohalee.hangastudio.core.bean.MultipartFilesBean;
import kr.sohalee.hangastudio.core.bean.PageNaviBean;
import kr.sohalee.hangastudio.core.exception.BizException;
import kr.sohalee.hangastudio.core.type.ExecutePreparable;
import kr.sohalee.hangastudio.core.util.BooleanUtil;
import kr.sohalee.hangastudio.core.util.JavaMailUtil;
import kr.sohalee.hangastudio.core.util.StringUtil;
import kr.sohalee.hangastudio.model.mail.MailStoreBean;
import kr.sohalee.hangastudio.model.mail.acc.AccRecvBean;
import kr.sohalee.hangastudio.model.mail.folder.FolderBean;
import kr.sohalee.hangastudio.model.mail.folder.FolderService;
import kr.sohalee.hangastudio.model.mail.message.MessageAttachBean;
import kr.sohalee.hangastudio.model.mail.message.MessageBean;
import kr.sohalee.hangastudio.model.mail.message.MessageService;
import kr.sohalee.hangastudio.model.mail.message.MessageUtil;
import kr.sohalee.hangastudio.presentation.service.mail.MailSupport;

@SuppressWarnings("serial")
public class MessageAction extends MailSupport implements ExecutePreparable {

	/*
	 * Service 셋팅 (Spring과 Struts2의 auto-wire)
	 */

	private FolderService folderService = null;

	/** Folder Service setter. Spring과 Struts2가 맞물려 자동으로 호출 */
	public void setMailFolderService(FolderService folderService)
			throws Exception {
		this.folderService = folderService;
	}

	private MessageService messageService = null;

	/** Folder Service setter. Spring과 Struts2가 맞물려 자동으로 호출 */
	public void setMailMessageService(MessageService messageService)
			throws Exception {
		this.messageService = messageService;
	}

	/*
	 * View에서 사용하는 Property
	 */

	/** 메일 폴더 목록 */
	private List<FolderBean> folderList = null;

	/** 메일 메세지 Bean */
	private MessageBean messageBean = null;

	/** 메일 메세지 목록 */
	private List<MessageBean> messageList = null;

	/** 검색 Bean */
	private MessageBean searchBean = null;

	/** 메세지의 첨부파일 Bean */
	private MessageAttachBean attachBean = null;

	/** 폼(multipart)으로 부터 전달된 파일 정보 */
	private MultipartFilesBean filesBean = null;

	/** 목록 등에서 체크박스로 체크된 목록 */
	private Integer[] messageNos = null;

	public List<FolderBean> getFolderList() {
		return folderList;
	}

	public void setFolderList(List<FolderBean> folderList) {
		this.folderList = folderList;
	}

	public MessageBean getMessageBean() {
		return messageBean;
	}

	public void setMessageBean(MessageBean messageBean) {
		this.messageBean = messageBean;
	}

	public List<MessageBean> getMessageList() {
		return messageList;
	}

	public void setMessageList(List<MessageBean> messageList) {
		this.messageList = messageList;
	}

	public MessageBean getSearchBean() {
		return searchBean;
	}

	public void setSearchBean(MessageBean searchBean) {
		this.searchBean = searchBean;
	}

	public MessageAttachBean getAttachBean() {
		return attachBean;
	}

	public void setAttachBean(MessageAttachBean attachBean) {
		this.attachBean = attachBean;
	}

	public MultipartFilesBean getFilesBean() {
		return filesBean;
	}

	public void setFilesBean(MultipartFilesBean filesBean) {
		this.filesBean = filesBean;
	}

	public Integer[] getMessageNos() {
		return messageNos;
	}

	public void setMessageNos(Integer[] messageNos) {
		this.messageNos = messageNos;
	}

	/*
	 * 사전작업
	 */

	/** ExecutePreparable 구현. 가장 마지막 Interceptor에서 호출 */
	public boolean executePrepare() throws Exception {

		// 받는 서버 기본 계정이라도 존재한다면 getSessAccRecvBean은 null이 될 수 없도록 만들었다.
		AccRecvBean recvBean = getSessAccRecvBean();
		if (recvBean == null) {
			addActionError("error.mail.recvDefaultNotFound");
			return false;
		}

		return true;
	}

	/*
	 * Action Execute Method
	 */

	private void validParamMessage() throws Exception {

		if (messageBean == null) {
			throw new BizException("error.mail.param.message");
		}
	}

	private void validParamFolder() throws Exception {

		validParamMessage();
		if (StringUtil.isEmpty(messageBean.getFolderName())) {
			throw new BizException("error.mail.param.folder.name");
		}
	}

	private void validParamNo() throws Exception {

		validParamFolder();
		if (messageBean.getNo() == null) {
			throw new BizException("error.mail.param.message.no");
		}
	}

	/**
	 * 메일 메시지 목록
	 * 
	 * @return String "success"
	 * @throws Exception
	 */
	public String executeList() throws Exception {

		validParamFolder();

		if (searchBean == null) {
			searchBean = new MessageBean();
		}
		searchBean.setFolderName(messageBean.getFolderName());

		// View 페이지 네비게이션용 값 저장
		if (pageNaviBean == null) {
			pageNaviBean = new PageNaviBean();
		}
		pageNaviBean.setItemTotalCnt(messageService.getMessageCount(
				getSessAccRecvBean(), searchBean));

		// 현재 페이지에 해당하는 자료의 시작번호와 끝번호를 계산하여 정의한다.
		MessageUtil.setPagingValue(searchBean, pageNaviBean.getCurrPageNo(),
				pageNaviBean.getItemCntPerPage(), pageNaviBean
						.getItemTotalCnt());

		messageList = messageService.getMessageList(getSessAccRecvBean(),
				searchBean);

		// 폴더 목록
		folderList = folderService.getFolderList(getSessAccRecvBean(), true,
				false);

		return SUCCESS;
	}

	/**
	 * 메일 메세지 보기
	 * 
	 * @return String "success"
	 * @throws Exception
	 */
	public String executeView() throws Exception {

		validParamMessage();

		messageBean = messageService.getMessage(getSessAccRecvBean(),
				messageBean);

		// 읽음 표시로 플래그를 바꿈
		messageBean.setRead(true);
		messageService.updateMessageReadFlag(getSessAccRecvBean(), messageBean);

		// 폴더 목록
		folderList = folderService.getFolderList(getSessAccRecvBean(), true,
				false);

		return SUCCESS;
	}

	/**
	 * 메일 메세지 전송 폼
	 * 
	 * @return String "success"
	 * @throws Exception
	 */
	public String executeSendForm() throws Exception {

		// 메일 발송시 보내는 또는 되받을 주소를 위해 기본 계정없이 사용할 수 없다.
		if (!validDefaultAccRecv()) {
			return ERROR;
		}

		return SUCCESS;
	}

	/** 메일 메세지 전송. 첨부파일이 있는 경우 첨부파일을 넣고, 인코딩을 셋팅하여 모델로 보낸다. */
	private void insertMessage() throws Exception {

		if (BooleanUtil.booleanValue(messageBean.getSave())) {
			messageBean.setSaveFolderName(getSessDefaultAccRecvBean()
					.getOutbox());
		}

		if (filesBean != null) {

			for (int i = 0; i < filesBean.getFiles().length; i++) {

				if (!StringUtil.isEmpty(filesBean.getFilesFileName()[i])) {

					File file = filesBean.getFiles()[i];

					MessageAttachBean bean = new MessageAttachBean();
					bean.setFilePath(file.getAbsolutePath());
					bean.setFileName(filesBean.getFilesFileName()[i]);

					messageBean.addAttachList(bean);

					bean = null;
					file = null;
				}
			}
		}

		messageBean.setFromName(getSessDefaultAccRecvBean().getId());
		messageBean
				.setFromAddress(getSessDefaultAccRecvBean().getAddressHost());
		// 메세지 생성시 멀티바이트가 들어가는 모든곳에 적용되는 인코딩
		messageBean.setEncoding(Constants.APP_ENCODING);
		messageService.insertMessage(getSessAccSendBean(),
				getSessAccRecvBean(), messageBean);

	}

	/**
	 * 메일 메세지 전송 처리 액션
	 * 
	 * @return String "success"
	 * @throws Exception
	 */
	public String executeSendAction() throws Exception {

		// 메일 발송시 보내는 또는 되받을 주소를 위해 기본 계정없이 사용할 수 없다.
		if (!validDefaultAccRecv()) {
			return ERROR;
		}

		// TODO Validation을 넣고 아래를 지우자
		validParamMessage();

		insertMessage();

		return SUCCESS;
	}

	private List<MessageBean> getMessageListFromMessageNos() throws Exception {

		List<MessageBean> messageList = new ArrayList<MessageBean>();
		for (Integer messageNo : messageNos) {

			MessageBean bean = new MessageBean();
			bean.setNo(messageNo);

			messageList.add(bean);

			bean = null;
		}

		return messageList;
	}

	private void deleteMessageList() throws Exception {

		messageBean.setChildMessageList(getMessageListFromMessageNos());

		String trash = getSessAccRecvBean().getTrash();
		// 휴지통이 지정되어 있지 않으면 완전히 삭제한다.
		if (StringUtil.isEmpty(trash)) {
			messageService.deleteMessageList(getSessAccRecvBean(), messageBean);
		}
		// 휴지통이 지정되어 있다면 휴지통으로 이동한다.
		else {

			// toName의 값을 넣은 뒤 folderService.getfolder를 하면 toSrc도 셋팅된다.
			messageBean.setSaveFolderName(trash);
			messageService.moveMessageList(getSessAccRecvBean(), messageBean);
		}

		trash = null;
	}

	/**
	 * 메일 메세지 삭제 처리 액션. (다중)
	 * 
	 * @return String "success"
	 * @throws Exception
	 */
	public String executeDeleteMultiAction() throws Exception {

		validParamFolder();

		if (messageNos != null) {
			deleteMessageList();
		}

		return SUCCESS;
	}

	/**
	 * 메일 메세지 삭제 처리 액션.
	 * 
	 * @return String "success"
	 * @throws Exception
	 */
	public String executeDeleteAction() throws Exception {

		validParamNo();

		messageNos = new Integer[1];
		messageNos[0] = messageBean.getNo();

		deleteMessageList();

		return SUCCESS;
	}

	private void moveMessageList() throws Exception {

		messageBean.setChildMessageList(getMessageListFromMessageNos());
		messageService.moveMessageList(getSessAccRecvBean(), messageBean);

		messageList = null;
	}

	/**
	 * 메일 메세지 이동 처리 액션. (다중)
	 * 
	 * @return String "success"
	 * @throws Exception
	 */
	public String executeMoveMultiAction() throws Exception {

		validParamFolder();

		if (messageNos != null) {
			moveMessageList();
		}

		return SUCCESS;
	}

	/**
	 * 메일 메세지 이동 처리 액션.
	 * 
	 * @return String "success"
	 * @throws Exception
	 */
	public String executeMoveAction() throws Exception {

		validParamNo();

		messageNos = new Integer[1];
		messageNos[0] = messageBean.getNo();

		moveMessageList();

		return SUCCESS;
	}

	/**
	 * 메일 메세지 답변 폼
	 * 
	 * @return String "success"
	 * @throws Exception
	 */
	public String executeReplyForm() throws Exception {

		// 메일 발송시 보내는 또는 되받을 주소를 위해 기본 계정없이 사용할 수 없다.
		if (!validDefaultAccRecv()) {
			return ERROR;
		}

		validParamNo();

		messageBean = messageService.getMessage(getSessAccRecvBean(),
				messageBean);
		if (messageBean == null) {
			addActionError("error.mail.notFound.message");
			return ERROR;
		}

		messageBean.setSubject("[RE] " + messageBean.getSubject());
		messageBean
				.setContent("\n\n\n--- Original Message ---------------------------------------\n\n"
						+ messageBean.getContent());
		messageBean.setTo(messageBean.getFromAddress());

		return SUCCESS;
	}

	/**
	 * 메일 메세지 전달 폼
	 * 
	 * @return String "success"
	 * @throws Exception
	 */
	public String executeForwardForm() throws Exception {

		// 메일 발송시 보내는 또는 되받을 주소를 위해 기본 계정없이 사용할 수 없다.
		if (!validDefaultAccRecv()) {
			return ERROR;
		}

		validParamNo();

		messageBean = messageService.getMessage(getSessAccRecvBean(),
				messageBean);
		if (messageBean == null) {
			addActionError("error.mail.notFound.message");
			return ERROR;
		}

		messageBean.setSubject("[FW] " + messageBean.getSubject());
		messageBean.setContent(null);
		messageBean.setTo(null);
		messageBean.setCc(null);
		messageBean.setBcc(null);

		return SUCCESS;
	}

	/**
	 * 메일 메세지 전달 처리 액션
	 * 
	 * @return String "success"
	 * @throws Exception
	 */
	public String executeForwardAction() throws Exception {

		// 메일 발송시 보내는 또는 되받을 주소를 위해 기본 계정없이 사용할 수 없다.
		if (!validDefaultAccRecv()) {
			return ERROR;
		}

		// TODO Validation을 넣고 아래를 지우자
		validParamNo();

		insertMessage();

		return SUCCESS;
	}

	/**
	 * 첨부파일 다운로드.<br />
	 * 서버에 파일 저장하지 않고 JavaMail의 getInputStream 사용. 메일서버가 리모트일경우 권장. 다운로드시 용량표시가
	 * 어려운 문제가 있음. JavaMail의 용량은 디코딩 전 용량.
	 * 
	 * @return String "success"
	 * @throws Exception
	 */
	public String executeAttachDownload() throws Exception {

		validParamNo();
		if (attachBean == null) {
			addActionError("error.mail.param.attach");
			return ERROR;
		}

		MailStoreBean storeBean = new MailStoreBean(getSessAccRecvBean());

		try {

			attachBean = messageService.getMessageAttach(getSessAccRecvBean(),
					messageBean, attachBean);

			HttpServletResponse response = getResponse();
			response.reset();

			/*
			 * attachBean.getSize로 구한것은 인코딩에 따라 용량이 달라진다. 파일로 먼저 저장한뒤에 다운로드 시키지
			 * 않는한 방법이 없는듯하다. 그러 파일로 저장한다면 IMAP에서 가져와 파일로 저장하고, 그걸 다시 클라이언트로 보내
			 * 비용이 발생한다.
			 */
			// response.setContentLength(attachBean.getSize());
			response.setContentType(attachBean.getFileType());
			response.setHeader("Content-Disposition", "attachment; filename="
					+ attachBean.getFileName() + ";");
			response.setHeader("Pragma", "no-cache;");
			response.setHeader("Expires", "-1;");

			// MessageAttachBean에 저장된 해당 BodyPart의 경로를 "|"로 잘라서 배열에 넣는다.
			String[] pathArray = attachBean.getMultipartPath().split("[|]");
			JavaMailUtil.attachPutOutputStream(storeBean.getStore(),
					messageBean.getFolderName(),
					messageBean.getNo().intValue(), pathArray, response
							.getOutputStream());
		} finally {
			storeBean = storeBean.close();
		}

		return SUCCESS;
	}
}
