/* 
 * This file is part of hanga studio.
 *
 * Copyright (C) 2007 hanga studio Project.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
package kr.sohalee.hangastudio.presentation.service.user;

import kr.sohalee.hangastudio.constants.ServiceConstants;
import kr.sohalee.hangastudio.core.exception.BizException;
import kr.sohalee.hangastudio.core.util.DateUtil;
import kr.sohalee.hangastudio.core.util.StringUtil;
import kr.sohalee.hangastudio.model.user.UserBean;
import kr.sohalee.hangastudio.model.user.UserService;
import kr.sohalee.hangastudio.presentation.service.ServiceSupport;

/**
 * 서비스 사용자 로그인 Action
 * 
 * @author <a href="mailto:sohalee@dreamwiz.com">Lee, Kyung-rok</a>
 * 
 */
@SuppressWarnings("serial")
public class UserAction extends ServiceSupport {

	private UserService userService = null;

	public void setUserService(UserService userService) {
		this.userService = userService;
	}

	/** 사용자 Bean */
	private UserBean userBean = null;

	public UserBean getUserBean() {
		return userBean;
	}

	public void setUserBean(UserBean userBean) {
		this.userBean = userBean;
	}

	public String executeLoginForm() throws Exception {
		return SUCCESS;
	}

	public String executeLoginAction() throws Exception {

		// 성공하든 실패하든 일단 여기까지 진입했으면 무조건 세션을 지워주기 위해 최상단에 선언
		clearSession();

		// 사용자가 없거나 패스워드가 틀리면 userService.login에서 Exception이 발생한다.
		try {
			userBean = userService.login(userBean);
		} catch (BizException e) {
			addActionError(e.getMessage());
			return INPUT;
		}

		clearSession();
		setSessionObject(ServiceConstants.SESS_USER, userBean);

		return SUCCESS;
	}

	public String executeInsertForm() throws Exception {
		return SUCCESS;
	}

	public String executeInsertAction() throws Exception {

		// TODO Validation을 넣고 아래를 지우자
		if (userBean == null || StringUtil.isEmpty(userBean.getUserId())) {
			addActionError("error.user.param.userId");
			return ERROR;
		}

		userBean.setAdminDate(null);
		userBean.setAdminId(null);
		userBean.setAdminIp(null);
		userBean.setModifyDate(null);
		userBean.setModifyIp(null);
		userBean.setWriteDate(DateUtil.getToday());
		userBean.setWriteIp(getRemoteAddr());
		userService.insertUser(userBean);
		
		return SUCCESS;
	}
}
